<?php
/**
 * Plugin Name:       Order Delivery Date & Time Scheduler
 * Plugin URI:        https://woocommerce.com/products/ka-order-delivery-date-and-time-scheduler/
 * Description:       Allows customers to pick a suitable date and time for order delivery and show estimated time of delivery.
 * Version:           1.0.1
 * Author:            KoalaApps
 * Developed By:      KoalaApps
 * Author URI:        https://woocommerce.com/vendor/koalaapps/
 * Support:           https://woocommerce.com/vendor/koalaapps/
 * License:           GNU General Public License v3.0
 * License URI:       http://www.gnu.org/licenses/gpl-3.0.html
 * Text Domain:       ka-order-delivery-date-and-time-scheduler
 * Requires Plugins: woocommerce
 * WC requires at least: 4.0
 * WC tested up to: 9.*.*
 * Requires at least: 6.5
 * Tested up to: 6.*.*
 * Requires PHP: 7.4
 */

if ( ! defined( 'WPINC' ) ) {
	die;
}


if ( ! class_exists( 'Ka_Order_Delivery_Date_And_Time_Scheduler' ) ) {

	class Ka_Order_Delivery_Date_And_Time_Scheduler {

		public function __construct() {

			$this->af_wc_daps_define_constants();

			add_action( 'init', array( $this, 'af_wc_daps_add_actions' ) );

			add_action( 'plugins_loaded', array( $this, 'af_wc_daps_init' ) );

			add_action( 'before_woocommerce_init', array( $this, 'af_wc_daps_hpos_compatibility' ) );

			register_activation_hook( __FILE__, array( $this, 'af_wc_daps_initial_settings' ) );

			add_action( 'woocommerce_blocks_loaded', array( $this, 'af_wc_daps_block_init' ) );
		}

		public function af_wc_daps_add_actions() {
			if ( defined( 'WC_PLUGIN_FILE' ) ) {

				add_action( 'after_setup_theme', array( $this, 'af_wc_daps_load_plugin_textdomain' ) );

				add_filter( 'woocommerce_product_data_tabs', array( $this, 'af_wc_daps_add_tab_to_product_data_tabs' ) );

				add_action( 'woocommerce_product_data_panels', array( $this, 'af_wc_daps_display_tab_content' ) );

				add_action( 'woocommerce_variation_options_pricing', array( $this, 'af_wc_daps_add_custom_content_to_variation' ), 10, 3 );

				add_action( 'woocommerce_save_product_variation', array( $this, 'af_wc_daps_save_custom_content' ), 10, 2 );

				add_filter( 'plugin_action_links_' . plugin_basename( __FILE__ ), array( $this, 'af_wc_daps_plugin_action_links' ) );

				add_filter( 'cron_schedules', array( $this, 'af_wc_daps_set_interval' ) );
				
				$this->af_wc_daps_register_remaining_orders_per_slot_lookup_event();
				
				add_action( 'af_wc_daps_update_slot_lookup_event', array( $this, 'af_wc_daps_update_slot_lookup_callback' ) );

				register_deactivation_hook( __FILE__, array( $this, 'af_wc_daps_plugin_deactivation_callback' ) );

				$this->af_wc_daps_holidays_custom_post_type();
				$this->af_wc_daps_special_days_custom_post_type();
				$this->af_wc_daps_delivery_and_time_slot_custom_post_type();
				$this->af_wc_daps_order_delivery_table_custom_post_type();

				include_once AF_WC_DAPS_PLUGIN_DIR . 'includes/af-wc-daps-general-functions.php';
				include_once AF_WC_DAPS_PLUGIN_DIR . 'includes/class-af-wc-daps-ajax-controller.php';
				if ( is_admin() ) {
					include_once AF_WC_DAPS_PLUGIN_DIR . 'includes/admin/class-af-wc-daps-admin.php';
				} else {
					include_once AF_WC_DAPS_PLUGIN_DIR . 'includes/front/class-af-wc-daps-front.php';
				}

				add_filter( 'post_updated_messages', array( $this, 'af_wc_daps_custom_post_published_messages' ));
				add_filter( 'bulk_post_updated_messages', array( $this, 'af_wc_daps_bulk_post_updated_messages' ), 10, 2 );
			}
		}


		public function af_wc_daps_init() {
			// Check the installation of WooCommerce module if it is not a multi site.
			if ( ! is_multisite() && ! in_array( 'woocommerce/woocommerce.php', apply_filters( 'active_plugins', get_option( 'active_plugins' ) ), true ) ) {

				add_action( 'admin_notices', array( $this, 'af_wc_daps_check_wocommerce' ) );
			}
		}

		public function af_wc_daps_block_init() {

			if ( ( $this->af_wc_daps_woocommerce_blocks_active() )&&( $this->af_wc_daps_woocommerce_blocks_version_supported() ) ) {
	
				require_once AF_WC_DAPS_PLUGIN_DIR . '/blocks-compatibility/af-wc-daps-checkout-block/af-wc-daps-block-integration-checkout.php';
	
	
				if ( class_exists( 'Af_Wc_Daps_Checkout_Blocks_Integration' ) ) {
	
					add_action(
						'woocommerce_blocks_checkout_block_registration',
						function ( $integration_registry ) {
							$integration_registry->register( new Af_Wc_Daps_Checkout_Blocks_Integration() );
						}
						);
	
				}
				
			}
		}

		public function af_wc_daps_plugin_activation_callback() {
		}

		public function af_wc_daps_register_remaining_orders_per_slot_lookup_event() {
			if ( ! wp_next_scheduled( 'af_wc_daps_update_slot_lookup_event' ) ) {
				wp_schedule_event( time(), 'af_wc_daps_interval', 'af_wc_daps_update_slot_lookup_event' );
			}
		}

		public function af_wc_daps_set_interval( $schedules ) {
			$schedules['af_wc_daps_interval'] = array(
				'interval' => 86400, 
				'display'  => 'Once a day', 
			);
			return $schedules;
		}

		public function af_wc_daps_plugin_deactivation_callback() {
			wp_clear_scheduled_hook( 'af_wc_daps_update_slot_lookup_event' );
		}

		public function af_wc_daps_woocommerce_blocks_active() {
			return class_exists( 'Automattic\WooCommerce\Blocks\Package' );
		}

		//function to update orders per slot data
		public function af_wc_daps_update_slot_lookup_callback() {
			$rule_ids =  get_posts(
				array(
					'post_type'      => 'daps_rules',
					'posts_per_page' => -1,
					'post_status'    => 'publish',
					'fields'         => 'ids',
					'orderby'        => 'menu_order',
					'order'          => 'DESC',
				)
			);


			$threshold_date = ( new DateTime('now') )->modify('-1 day');

			foreach ($rule_ids as $rule_id) {
				$time_slot_select_data = get_post_meta($rule_id, 'af_wc_daps_time_select_slot_per_order_details', true);
				$time_slot_data        = get_post_meta($rule_id, 'af_wc_daps_time_slot_per_order_details', true);

				foreach ($time_slot_select_data as $key => $value) {
					$date = DateTime::createFromFormat('Y-m-d', explode('_', $key)[0]);
					if ($date < $threshold_date) {
						unset($time_slot_select_data[ $key ]);
					} 
				}

				foreach ($time_slot_data as $key => $value) {
					$date = DateTime::createFromFormat('Y-m-d', explode('_', $key)[0]);
					if ($date < $threshold_date) {
						unset($time_slot_data[ $key ]);
					} 
				}

				update_post_meta($rule_id, 'af_wc_daps_time_select_slot_per_order_details', $time_slot_select_data);
				update_post_meta($rule_id, 'af_wc_daps_time_slot_per_order_details', $time_slot_data);
			}


			$product_ids = get_posts(array(
				'post_type'      => array( 'product', 'product_variation' ), 
				'posts_per_page' => -1,
				'fields'         => 'ids',
			));
						
			foreach ($product_ids as $product_id) {
				$time_slot_select_data = get_post_meta($product_id, 'af_wc_daps_time_select_slot_per_order_details', true);
				$time_slot_data        = get_post_meta($product_id, 'af_wc_daps_time_slot_per_order_details', true);
			
				if (empty($time_slot_select_data) && empty($time_slot_data)) {
					continue;
				}
			
				foreach ($time_slot_select_data as $key => $value) {
					$date = DateTime::createFromFormat('Y-m-d', explode('_', $key)[0]);
					if ($date < $threshold_date) {
						unset($time_slot_select_data[ $key ]);
					}
				}
			
				foreach ($time_slot_data as $key => $value) {
					$date = DateTime::createFromFormat('Y-m-d', explode('_', $key)[0]);
					if ($date < $threshold_date) {
						unset($time_slot_data[ $key ]);
					}
				}

				update_post_meta($product_id, 'af_wc_daps_time_select_slot_per_order_details', $time_slot_select_data);
				update_post_meta($product_id, 'af_wc_daps_time_slot_per_order_details', $time_slot_data);
			}
		}
	
		public function af_wc_daps_woocommerce_blocks_version_supported() {
			return version_compare(
				\Automattic\WooCommerce\Blocks\Package::get_version(),
				'7.3.0',
				'>='
			);
		}

		public function af_wc_daps_check_wocommerce() {
			deactivate_plugins( __FILE__ );
			?>
			<div id="message" class="error">
				<p>
					<strong>
						<?php esc_html_e( 'Order Delivery Date & Time Scheduler plugin is inactive. WooCommerce plugin must be active in order to activate it.', 'ka-order-delivery-date-and-time-scheduler' ); ?>
					</strong>
				</p>
			</div>
			<?php
		}



		public function af_wc_daps_hpos_compatibility() {
			if ( class_exists( \Automattic\WooCommerce\Utilities\FeaturesUtil::class ) ) {
				\Automattic\WooCommerce\Utilities\FeaturesUtil::declare_compatibility( 'custom_order_tables', __FILE__, true );
			}
		}


		public function af_wc_daps_define_constants() {
			if ( ! defined( 'AF_WC_DAPS_BLOCK_VERSION' ) ) {
				$af_wc_daps_block_version = get_file_data( __FILE__, array( 'version' => 'version' ) );
				define( 'AF_WC_DAPS_BLOCK_VERSION', $af_wc_daps_block_version['version'] );  
			}

			if ( ! defined( 'AF_WC_DAPS_PLUGIN_DIR' ) ) {
				define( 'AF_WC_DAPS_PLUGIN_DIR', plugin_dir_path( __FILE__ ) );
			}
			if ( ! defined( 'AF_WC_DAPS_PLUGIN_URL' ) ) {
				define( 'AF_WC_DAPS_PLUGIN_URL', plugin_dir_url( __FILE__ ) );
			}
		}


		public function af_wc_daps_load_plugin_textdomain() {
			if ( function_exists( 'load_plugin_textdomain' ) ) {
				load_plugin_textdomain( 'ka-order-delivery-date-and-time-scheduler', false, AF_WC_DAPS_PLUGIN_DIR . '/languages/' );
			}
		}

		public function af_wc_daps_plugin_action_links( $actions ) {
			$af_wc_daps_custom_actions = array(
				'settings' => sprintf( '<a href="%s">%s</a>', admin_url( 'admin.php?page=wc-settings&tab=settings_tab_order_delivery' ), __( 'Settings', 'ka-order-delivery-date-and-time-scheduler' ) ),
			);
			return array_merge( $af_wc_daps_custom_actions, $actions );
		}

		public function af_wc_daps_initial_settings() {
			$general_settings = array(
				'af_wc_daps_enable_at_single_product_page' => '1',
				'af_wc_daps_enable_at_checkout_page'       => '1',
				'af_wc_daps_delivery_info_field_position'  => 'woocommerce_before_checkout_billing_form',
				'af_wc_daps_mandatory_product_timeslot_field' => '1',
				'af_wc_daps_mandatory_order_timeslot_field' => '1',
			);

			$calendar_settings = array(
				'af_wc_daps_admin_calendar_theme'       => 'base',
				'af_wc_daps_front_calendar_theme'       => 'base',
				'af_wc_daps_available_dates_background_color' => '#00cc66',
				'af_wc_daps_holiday_dates_background_color' => '#ff0000',
				'af_wc_daps_special_dates_background_color' => '#151df9',
				'af_wc_daps_disabled_dates_background_color' => '#d0d0e1',
				'af_wc_daps_available_dates_text_color' => '#ffffff',
				'af_wc_daps_holiday_dates_text_color'   => '#ffffff',
				'af_wc_daps_special_dates_text_color'   => '#ffffff',
				'af_wc_daps_disabled_dates_text_color'  => 'ffffff',
			);

			$expected_info_settings = array(
				'af_wc_daps_enable_icon_for_expected_date_info'     => '1',
				'af_wc_daps_bg_color_of_expected_date_container'    => '#EFF3F5',
				'af_wc_daps_text_color_of_expected_date_container'  => '#2E2E2E',
			);

			$timezone_settings = array(
				'af_wc_daps_use_default_timezone' => '1',
			);

			$existing_general_settings       = get_option('af_wc_daps_general_settings')?get_option('af_wc_daps_general_settings'):array();
			$existing_expected_info_settings = get_option('af_wc_daps_expected_info_style_settings')?get_option('af_wc_daps_expected_info_style_settings'):array();
			$existing_timezone_settings      = get_option('af_wc_daps_timezone_settings')?get_option('af_wc_daps_timezone_settings'):array();

			update_option('af_wc_daps_general_settings', array_merge($existing_general_settings, $general_settings));
			update_option('af_wc_daps_calendar_style_settings', $calendar_settings);
			update_option('af_wc_daps_expected_info_style_settings', array_merge($existing_expected_info_settings, $expected_info_settings));
			update_option('af_wc_daps_timezone_settings', array_merge($existing_timezone_settings, $timezone_settings));
		}


		public function af_wc_daps_holidays_custom_post_type() {
			$labels = array(
				'name'                => esc_html__( 'Holidays', 'ka-order-delivery-date-and-time-scheduler' ),
				'singular_name'       => esc_html__( 'Holiday', 'ka-order-delivery-date-and-time-scheduler' ),
				'add_new'             => esc_html__( 'Add New  Holiday', 'ka-order-delivery-date-and-time-scheduler' ),
				'add_new_item'        => esc_html__( 'Add New Holiday', 'ka-order-delivery-date-and-time-scheduler' ),
				'edit_item'           => esc_html__( 'Edit Holiday', 'ka-order-delivery-date-and-time-scheduler' ),
				'new_item'            => esc_html__( 'New Holiday', 'ka-order-delivery-date-and-time-scheduler' ),
				'view_item'           => esc_html__( 'View Holiday', 'ka-order-delivery-date-and-time-scheduler' ),
				'search_items'        => esc_html__( 'Search Holidays', 'ka-order-delivery-date-and-time-scheduler' ),
				'exclude_from_search' => true,
				'not_found'           => esc_html__( 'No Holidays found', 'ka-order-delivery-date-and-time-scheduler' ),
				'not_found_in_trash'  => esc_html__( 'No Holidays found in trash', 'ka-order-delivery-date-and-time-scheduler' ),
				'parent_item_colon'   => '',
				'all_items'           => esc_html__( 'Delivery Date & Time', 'ka-order-delivery-date-and-time-scheduler' ),
				'menu_name'           => esc_html__( 'Delivery Date & Time', 'ka-order-delivery-date-and-time-scheduler' ),
			);

			$args = array(
				'labels'             => $labels,
				'public'             => false,
				'publicly_queryable' => true,
				'show_ui'            => true,
				'show_in_menu'       => 'woocommerce',
				'query_var'          => true,
				'rewrite'            => true,
				'capability_type'    => 'post',
				'has_archive'        => true,
				'hierarchical'       => false,
				'menu_position'      => 30,
				'supports'           => array( 'title' ),
			);

			register_post_type( 'daps_holidays', $args );
		}

		public function af_wc_daps_special_days_custom_post_type() {
			$labels = array(
				'name'                => esc_html__( 'Special Days', 'ka-order-delivery-date-and-time-scheduler' ),
				'singular_name'       => esc_html__( 'Special day', 'ka-order-delivery-date-and-time-scheduler' ),
				'add_new'             => esc_html__( 'Add New  Special day', 'ka-order-delivery-date-and-time-scheduler' ),
				'add_new_item'        => esc_html__( 'Add New Special day', 'ka-order-delivery-date-and-time-scheduler' ),
				'edit_item'           => esc_html__( 'Edit Special day', 'ka-order-delivery-date-and-time-scheduler' ),
				'new_item'            => esc_html__( 'New Special day', 'ka-order-delivery-date-and-time-scheduler' ),
				'view_item'           => esc_html__( 'View Special day', 'ka-order-delivery-date-and-time-scheduler' ),
				'search_items'        => esc_html__( 'Search Special days', 'ka-order-delivery-date-and-time-scheduler' ),
				'exclude_from_search' => true,
				'not_found'           => esc_html__( 'No Special days found', 'ka-order-delivery-date-and-time-scheduler' ),
				'not_found_in_trash'  => esc_html__( 'No Special days found in trash', 'ka-order-delivery-date-and-time-scheduler' ),
				'parent_item_colon'   => '',
				'all_items'           => esc_html__( 'Delivery Date & Time', 'ka-order-delivery-date-and-time-scheduler' ),
				'menu_name'           => esc_html__( 'Delivery Date & Time', 'ka-order-delivery-date-and-time-scheduler' ),

			);

			$args = array(
				'labels'             => $labels,
				'public'             => false,
				'publicly_queryable' => true,
				'show_ui'            => true,
				'show_in_menu'       => 'woocommerce',
				'query_var'          => true,
				'rewrite'            => true,
				'capability_type'    => 'post',
				'has_archive'        => true,
				'hierarchical'       => false,
				'menu_position'      => 30,
				'supports'           => array( 'title' ),
			);

			register_post_type( 'daps_special_days', $args );
		}

		


		public function af_wc_daps_delivery_and_time_slot_custom_post_type() {
			$labels = array(
				'name'                => esc_html__( 'Rules', 'ka-order-delivery-date-and-time-scheduler' ),
				'singular_name'       => esc_html__( 'Rules', 'ka-order-delivery-date-and-time-scheduler' ),
				'add_new'             => esc_html__( 'Add New Rule', 'ka-order-delivery-date-and-time-scheduler' ),
				'add_new_item'        => esc_html__( 'Add New Rule', 'ka-order-delivery-date-and-time-scheduler' ),
				'edit_item'           => esc_html__( 'Edit Rule', 'ka-order-delivery-date-and-time-scheduler' ),
				'new_item'            => esc_html__( 'New Rule ', 'ka-order-delivery-date-and-time-scheduler' ),
				'view_item'           => esc_html__( 'View Rules', 'ka-order-delivery-date-and-time-scheduler' ),
				'search_items'        => esc_html__( 'Search Rules', 'ka-order-delivery-date-and-time-scheduler' ),
				'exclude_from_search' => true,
				'not_found'           => esc_html__( 'No Rule found', 'ka-order-delivery-date-and-time-scheduler' ),
				'not_found_in_trash'  => esc_html__( 'No Rule found in trash', 'ka-order-delivery-date-and-time-scheduler' ),
				'parent_item_colon'   => '',
				'all_items'           => esc_html__( 'Delivery Date & Time', 'ka-order-delivery-date-and-time-scheduler' ),
				'menu_name'           => esc_html__( 'Delivery Date & Time', 'ka-order-delivery-date-and-time-scheduler' ),
				'attributes'          => esc_html__( 'Rule Priority', 'ka-order-delivery-date-and-time-scheduler' ),

			);

			$args = array(
				'labels'             => $labels,
				'public'             => false,
				'publicly_queryable' => true,
				'show_ui'            => true,
				'show_in_menu'       => 'woocommerce',
				'query_var'          => true,
				'rewrite'            => true,
				'capability_type'    => 'post',
				'has_archive'        => true,
				'hierarchical'       => false,
				'menu_position'      => 30,
				'supports'           => array( 'title', 'page-attributes' ),
			);

			register_post_type( 'daps_rules', $args );
		}


		public function af_wc_daps_order_delivery_table_custom_post_type() {
			$labels = array(
				'name'               => esc_html__( 'Order Deliveries', 'ka-order-delivery-date-and-time-scheduler' ),
				'singular_name'      => esc_html__( 'Order Deliveries', 'ka-order-delivery-date-and-time-scheduler' ),
				'not_found'          => esc_html__( 'No order deliveries found', 'ka-order-delivery-date-and-time-scheduler' ),
				'not_found_in_trash' => esc_html__( 'No order deliveries found in trash', 'ka-order-delivery-date-and-time-scheduler' ),
				'parent_item_colon'  => '',
				'all_items'          => esc_html__( 'Delivery Date & Time', 'ka-order-delivery-date-and-time-scheduler' ),
				'menu_name'          => esc_html__( 'Delivery Date & Time', 'ka-order-delivery-date-and-time-scheduler' ),

			);

			$args = array(
				'labels'             => $labels,
				'public'             => false,
				'publicly_queryable' => true,
				'show_ui'            => true,
				'query_var'          => true,
				'capability_type'    => 'post',
				'has_archive'        => true,
				'menu_position'      => 30,
				'show_in_menu'       => 'woocommerce',
				'map_meta_cap'       => true,
				'capabilities'       => array(
					'create_posts' => 'do_not_allow',
				),
			);

			register_post_type( 'daps_order_delivery', $args );
		}

		public function af_wc_daps_add_tab_to_product_data_tabs( $tabs ) {
			global $post;
			
			if ( $post && 'product' === $post->post_type && ( wc_get_product( $post->ID )->is_type( 'variable' ) || wc_get_product( $post->ID )->is_type( 'external' ) || wc_get_product( $post->ID )->is_type( 'grouped' ) || wc_get_product( $post->ID )->is_downloadable() || wc_get_product( $post->ID )->is_virtual() ) ) {
				return $tabs;
			}

			$tabs['af_wc_daps_rules'] = array(
				'label'    => esc_html__( 'Order Delivery Date & Time Scheduler', 'ka-order-delivery-date-and-time-scheduler' ),
				'target'   => 'af_wc_daps_rules_tab',
				'priority' => 21,
			);

			return $tabs;
		}


		public function af_wc_daps_display_tab_content() {
			global $post;
			$post_id = $post->ID;

			if ( $post && 'product' === $post->post_type && wc_get_product( $post_id )->is_type( 'variable' ) ) {
				return;
			}

			?>
			<div class="panel woocommerce_options_panel af-wc-daps-product-level-setting-container">
				<button id='af-wc-daps-product-level-general-setting-btn' class="tab-button-product-level">General Settings</button>
				<button id='af-wc-daps-product-level-time-slot-setting-btn' class="tab-button-product-level">Time Slots</button>
				<div id="af-wc-daps-product-level-rule-settings">
					<?php
					af_wc_daps_rule_advance_settings( $post );
					?>

				</div>
				<div id="af-wc-daps-product-level-time-slot-settings" style="display:none">
					<?php
						af_wc_daps_time_slot_section_content($post_id);
					?>
			
				</div>
			</div>
			<?php
		}

		public function af_wc_daps_add_custom_content_to_variation( $loop, $variation_data, $variation ) {

			?>
			<div>
				<button id="af-wc-daps-general-settings-btn-<?php echo esc_attr($variation->ID); ?>" class='af-wc-daps-variation-level-general-setting-btn tab-button-product-level' data-variation-id=<?php echo esc_attr($variation->ID); ?>>General Settings</button>
				<button id="af-wc-daps-time-slot-settings-btn-<?php echo esc_attr($variation->ID); ?>" class='af-wc-daps-variation-level-time-slot-setting-btn tab-button-product-level' data-variation-id=<?php echo esc_attr($variation->ID); ?>>Time Slots</button>
				<div class="af-wc-daps-variation-level-rule-settings-<?php echo esc_attr($variation->ID); ?>">
					<?php
				af_wc_daps_rule_advance_settings_for_product_variation( $variation->ID );
					?>

				</div>
				<div class="af-wc-daps-variation-level-time-slot-settings-<?php echo esc_attr($variation->ID); ?>" style="display:none">
					<?php
						af_wc_daps_time_slot_section_content_for_product_variation($variation->ID);
					?>
			
				</div>
			</div>
		
			<?php
		}

		public function af_wc_daps_save_custom_content( $variation_id, $loop ) {

			$nonce = isset( $_POST['af_wc_daps_nonce_field'] ) ? sanitize_text_field( wp_unslash( $_POST['af_wc_daps_nonce_field'] ) ) : '';

			if ( isset( $_POST['af_wc_daps_nonce_field'] ) && ! wp_verify_nonce( $nonce, 'af_wc_daps_nonce' ) ) {
				wp_die( esc_html__( 'Failed security check!', 'ka-order-delivery-date-and-time-scheduler' ) );
			}

			$af_wc_daps_enable_rule_name                      = 'af_wc_daps_enable_rule_' . $variation_id;
			$af_wc_daps_rule_delivery_selection_mode_name     = 'af_wc_daps_rule_delivery_selection_mode_' . $variation_id;
			$af_wc_daps_rule_minimum_delivery_days_name       = 'af_wc_daps_rule_minimum_delivery_days_' . $variation_id;
			$af_wc_daps_rule_maximum_delivery_days_name       = 'af_wc_daps_rule_maximum_delivery_days_' . $variation_id;
			$af_wc_daps_rule_expected_date_message_text_name  = 'af_wc_daps_rule_expected_date_message_text_' . $variation_id;
			$af_wc_daps_rule_min_available_days_name          = 'af_wc_daps_rule_min_available_days_' . $variation_id;
			$af_wc_daps_rule_max_available_days_name          = 'af_wc_daps_rule_max_available_days_' . $variation_id;
			$af_wc_daps_rule_available_days_of_week_name      = 'af_wc_daps_rule_available_days_of_week_' . $variation_id;
			$af_wc_daps_rule_max_orders_name                  = 'af_wc_daps_rule_max_orders_' . $variation_id;
			$af_wc_daps_rule_time_selection_type_name         = 'af_wc_daps_rule_time_selection_type_' . $variation_id;
			$af_wc_daps_rule_time_selector_start_time_name    = 'af_wc_daps_rule_time_selector_start_time_' . $variation_id;
			$af_wc_daps_rule_time_selector_end_time_name      = 'af_wc_daps_rule_time_selector_end_time_' . $variation_id;
			$af_wc_daps_rule_time_selector_slot_duration_name = 'af_wc_daps_rule_time_selector_slot_duration_' . $variation_id;
			$orders_per_slot_name                             = 'orders_per_slot_' . $variation_id;


			$af_wc_daps_rule_week_day_fee_name = 'af_wc_daps_rule_week_day_fee_' . $variation_id;

			$af_wc_daps_time_slot_settings_name = 'af_wc_daps_time_slot_settings_' . $variation_id;
			$af_wc_daps_row_counter_name        = 'af_wc_daps_row_counter_' . $variation_id;

			$af_wc_daps_enable_rule                     = isset( $_POST[ $af_wc_daps_enable_rule_name ] ) ? sanitize_text_field( wp_unslash( $_POST[ $af_wc_daps_enable_rule_name ] ) ) : '';
			$af_wc_daps_rule_delivery_selection_mode    = isset( $_POST[ $af_wc_daps_rule_delivery_selection_mode_name ] ) ? sanitize_text_field( wp_unslash( $_POST[ $af_wc_daps_rule_delivery_selection_mode_name ] ) ) : '';
			$af_wc_daps_rule_minimum_delivery_days      = isset( $_POST[ $af_wc_daps_rule_minimum_delivery_days_name ] ) ? sanitize_text_field( wp_unslash( $_POST[ $af_wc_daps_rule_minimum_delivery_days_name ] ) ) : '';
			$af_wc_daps_rule_maximum_delivery_days      = isset( $_POST[ $af_wc_daps_rule_maximum_delivery_days_name ] ) ? sanitize_text_field( wp_unslash( $_POST[ $af_wc_daps_rule_maximum_delivery_days_name ] ) ) : '';
			$af_wc_daps_rule_expected_date_message_text = isset( $_POST[ $af_wc_daps_rule_expected_date_message_text_name ] ) ? sanitize_text_field( wp_unslash( $_POST[ $af_wc_daps_rule_expected_date_message_text_name ] ) ) : '';
			$af_wc_daps_rule_min_available_days         = isset( $_POST[ $af_wc_daps_rule_min_available_days_name ] ) ? sanitize_text_field( wp_unslash( $_POST[ $af_wc_daps_rule_min_available_days_name ] ) ) : '';
			$af_wc_daps_rule_max_available_days         = isset( $_POST[ $af_wc_daps_rule_max_available_days_name ] ) ? sanitize_text_field( wp_unslash( $_POST[ $af_wc_daps_rule_max_available_days_name ] ) ) : '';
			
		
			if (isset($_POST[ $af_wc_daps_rule_available_days_of_week_name ])) {
				$af_wc_daps_rule_available_days_of_week = sanitize_meta('', wp_unslash($_POST[ $af_wc_daps_rule_available_days_of_week_name ]), '');
			} else {
				$af_wc_daps_rule_available_days_of_week = array();
			}

			$af_wc_daps_rule_max_orders = isset( $_POST[ $af_wc_daps_rule_max_orders_name ] ) ? sanitize_text_field( wp_unslash( $_POST[ $af_wc_daps_rule_max_orders_name ] ) ) : '';

			$af_wc_daps_rule_time_selection_type = isset( $_POST[ $af_wc_daps_rule_time_selection_type_name ] ) ? sanitize_text_field( wp_unslash( $_POST[ $af_wc_daps_rule_time_selection_type_name ] ) ) : '';

			$af_wc_daps_rule_time_selector_start_time    = isset( $_POST[ $af_wc_daps_rule_time_selector_start_time_name ] ) ? sanitize_text_field( wp_unslash( $_POST[ $af_wc_daps_rule_time_selector_start_time_name ] ) ) : '';
			$af_wc_daps_rule_time_selector_end_time      = isset( $_POST[ $af_wc_daps_rule_time_selector_end_time_name ] ) ? sanitize_text_field( wp_unslash( $_POST[ $af_wc_daps_rule_time_selector_end_time_name ] ) ) : '';
			$af_wc_daps_rule_time_selector_slot_duration = isset( $_POST[ $af_wc_daps_rule_time_selector_slot_duration_name ] ) ? sanitize_text_field( wp_unslash( $_POST[ $af_wc_daps_rule_time_selector_slot_duration_name ] ) ) : '';
			$orders_per_slot                             = isset( $_POST[ $orders_per_slot_name ] ) ? sanitize_text_field( wp_unslash( $_POST[ $orders_per_slot_name ] ) ) : '';

			//handling per order limit for time select slot
			$already_stored_rule_settings = get_post_meta($variation_id, 'af_wc_daps_rule_settings', true);

			if (is_array($already_stored_rule_settings) && !empty($already_stored_rule_settings)) {
				
				$current_time_selector_slot_duration = '' != $af_wc_daps_rule_time_selector_slot_duration ? $af_wc_daps_rule_time_selector_slot_duration:'30';
				$start_time                          = isset($already_stored_rule_settings['af_wc_daps_rule_time_selector_start_time'])?$already_stored_rule_settings['af_wc_daps_rule_time_selector_start_time']:'';
				$end_time                            = isset($already_stored_rule_settings['af_wc_daps_rule_time_selector_end_time'])?$already_stored_rule_settings['af_wc_daps_rule_time_selector_end_time']:'';
				$slot_duration                       = isset($already_stored_rule_settings['af_wc_daps_rule_time_selector_slot_duration']) && '' != $already_stored_rule_settings['af_wc_daps_rule_time_selector_slot_duration'] ?$already_stored_rule_settings['af_wc_daps_rule_time_selector_slot_duration']:'30';

				$old_orders_per_slot = isset($already_stored_rule_settings['orders_per_slot']) && '' != $already_stored_rule_settings['orders_per_slot'] ?$already_stored_rule_settings['orders_per_slot']:0;

				//if any of these values changes then reset the slot data and per order count
				if ($start_time != $af_wc_daps_rule_time_selector_start_time || $end_time != $af_wc_daps_rule_time_selector_end_time || ( $slot_duration != $current_time_selector_slot_duration )) {
					update_post_meta($variation_id, 'af_wc_daps_time_select_slot_per_order_details', array());
				}


				if ('' == $orders_per_slot) {
					update_post_meta($variation_id, 'af_wc_daps_time_select_slot_per_order_details', array());

				} elseif ($old_orders_per_slot != $orders_per_slot) { 
					$af_wc_daps_time_select_slot_per_order_details = get_post_meta($variation_id, 'af_wc_daps_time_select_slot_per_order_details', true);

					if (is_array($af_wc_daps_time_select_slot_per_order_details) && !empty($af_wc_daps_time_select_slot_per_order_details)) {
						foreach ($af_wc_daps_time_select_slot_per_order_details as $key =>  $value ) {
							$new_value = $value + $orders_per_slot - $old_orders_per_slot;
							$af_wc_daps_time_select_slot_per_order_details[ $key ] = $new_value < 0 ? 0 : $new_value;
						}

						update_post_meta($variation_id, 'af_wc_daps_time_select_slot_per_order_details', $af_wc_daps_time_select_slot_per_order_details);
					}
				} 
			}


			$af_wc_daps_rule_week_day_fee = sanitize_meta( '', wp_unslash( isset( $_POST[ $af_wc_daps_rule_week_day_fee_name ] ) ? $_POST[ $af_wc_daps_rule_week_day_fee_name ] : array() ), '' );

			$af_wc_daps_time_slot_settings = sanitize_meta( '', wp_unslash( isset( $_POST[ $af_wc_daps_time_slot_settings_name ] ) ? $_POST[ $af_wc_daps_time_slot_settings_name ] : array() ), '' );

			//handling per order slot change for time slot
			$already_stored_timeslot_settings = get_post_meta($variation_id, 'af_wc_daps_time_slot_settings', true);

			foreach ($af_wc_daps_time_slot_settings as $actual_key=> $time_slot) {
				$orders_per_slot = isset($time_slot['orders_per_slot'])? $time_slot['orders_per_slot']:'';

				$old_orders_per_slot = isset($already_stored_timeslot_settings[ $actual_key ]['orders_per_slot']) && '' != $already_stored_timeslot_settings[ $actual_key ]['orders_per_slot'] ?$already_stored_timeslot_settings[ $actual_key ]['orders_per_slot']:0;

				$af_wc_daps_time_slot_per_order_details = get_post_meta($variation_id, 'af_wc_daps_time_slot_per_order_details', true);

				if (is_array($af_wc_daps_time_slot_per_order_details) && !empty($af_wc_daps_time_slot_per_order_details)) {
					foreach ($af_wc_daps_time_slot_per_order_details as $key =>  $value ) {

						if (preg_match('/[^_]+$/', $key, $matches)) {
							$timeslot_key = $matches[0];

							if ($actual_key == $timeslot_key ) {
								
								if ( '' == $orders_per_slot) {
									unset($af_wc_daps_time_slot_per_order_details[ $key ]);
								} elseif ($old_orders_per_slot != $orders_per_slot) { 
									$new_value                                      = $value + $orders_per_slot - $old_orders_per_slot;
									$af_wc_daps_time_slot_per_order_details[ $key ] = $new_value>0?$new_value:0;
								}
							}
						}
					}

					update_post_meta($variation_id, 'af_wc_daps_time_slot_per_order_details', $af_wc_daps_time_slot_per_order_details);
				}
			}












			$af_wc_daps_row_counter = isset( $_POST[ $af_wc_daps_row_counter_name ] ) ? sanitize_text_field( wp_unslash( $_POST[ $af_wc_daps_row_counter_name ] ) ) : '';


			$af_wc_daps_rule_settings = array(
				'af_wc_daps_enable_rule'                   => $af_wc_daps_enable_rule,
				'af_wc_daps_selected_product_ids'          => $variation_id,
				'af_wc_daps_product_ids_for_rule'          => $variation_id,
				'af_wc_daps_rule_delivery_selection_mode'  => $af_wc_daps_rule_delivery_selection_mode,
				'af_wc_daps_rule_minimum_delivery_days'    => $af_wc_daps_rule_minimum_delivery_days,
				'af_wc_daps_rule_maximum_delivery_days'    => $af_wc_daps_rule_maximum_delivery_days,
				'af_wc_daps_rule_expected_date_message_text' => $af_wc_daps_rule_expected_date_message_text,
				'af_wc_daps_rule_min_available_days'       => $af_wc_daps_rule_min_available_days,
				'af_wc_daps_rule_max_available_days'       => $af_wc_daps_rule_max_available_days,
				'af_wc_daps_rule_available_days_of_week'   => $af_wc_daps_rule_available_days_of_week,
				'af_wc_daps_rule_max_orders'               => $af_wc_daps_rule_max_orders,
				'af_wc_daps_rule_time_selection_type'      => $af_wc_daps_rule_time_selection_type,
				'af_wc_daps_rule_time_selector_start_time' => $af_wc_daps_rule_time_selector_start_time,
				'af_wc_daps_rule_time_selector_end_time'   => $af_wc_daps_rule_time_selector_end_time,
				'af_wc_daps_rule_time_selector_slot_duration' => $af_wc_daps_rule_time_selector_slot_duration,
				'orders_per_slot'                          => $orders_per_slot,
				'af_wc_daps_rule_week_day_fee'             => $af_wc_daps_rule_week_day_fee,
			);

			update_post_meta( $variation_id, 'af_wc_daps_rule_settings', $af_wc_daps_rule_settings );

			update_post_meta($variation_id , 'af_wc_daps_time_slot_settings', $af_wc_daps_time_slot_settings);

			update_post_meta($variation_id, 'af_wc_daps_row_counter', $af_wc_daps_row_counter);
		}

		public function af_wc_daps_custom_post_published_messages( $custom_messages ) {

			global $post, $post_ID;

			$post_types_array = array( 'daps_rules', 'daps_special_days', 'daps_holidays' );


			if ( !in_array($post->post_type, $post_types_array) ) {

				return $custom_messages;
			}


			if ('daps_rules' == $post->post_type) {

				// Define your custom post type messages
				$custom_post_type               = 'daps_rules'; // Replace with your actual post type name
				$custom_post_type_singular_name = 'Rule'; // Replace with singular name of your post type
				$custom_post_type_plural_name   = 'Rules'; // Replace with plural name of your post type

			} elseif ('daps_holidays' == $post->post_type) {

				// Define your custom post type messages
				$custom_post_type               = 'daps_holidays'; // Replace with your actual post type name
				$custom_post_type_singular_name = 'Holiday'; // Replace with singular name of your post type
				$custom_post_type_plural_name   = 'Holidays'; // Replace with plural name of your post type

			} elseif ('daps_special_days' == $post->post_type) {

				// Define your custom post type messages
				$custom_post_type               = 'daps_special_days'; // Replace with your actual post type name
				$custom_post_type_singular_name = 'Special Day'; // Replace with singular name of your post type
				$custom_post_type_plural_name   = 'Special Days'; // Replace with plural name of your post type

			}


			$custom_messages[ $custom_post_type ] = array(
				0  => '', // Unused. Messages start at index 1.
				/* translators: %s: custom post update */
				1  => sprintf( __( '%s updated.', 'ka-order-delivery-date-and-time-scheduler' ), $custom_post_type_singular_name ),
				2  => __( 'Custom field updated.', 'ka-order-delivery-date-and-time-scheduler' ),
				3  => __( 'Custom field deleted.', 'ka-order-delivery-date-and-time-scheduler' ),
				/* translators: %s: custom post update */
				4  => sprintf( __( '%s updated.', 'ka-order-delivery-date-and-time-scheduler' ), $custom_post_type_singular_name ),
				/* translators: %s: custom post revision */
				5  => isset( $_GET['revision'] ) ? sprintf( __( '%1$s restored to revision from %2$s', 'ka-order-delivery-date-and-time-scheduler' ), $custom_post_type_singular_name, wp_post_revision_title( (int) $_GET['revision'], false ) ) : false,
				/* translators: %s: custom post published */
				6  => sprintf( __( '%s published.', 'ka-order-delivery-date-and-time-scheduler' ), $custom_post_type_singular_name ),
				/* translators: %s: custom post saved */
				7  => sprintf( __( '%s saved.', 'ka-order-delivery-date-and-time-scheduler' ), $custom_post_type_singular_name ),
				/* translators: %s: custom post submitted */
				8  => sprintf( __( '%s submitted.', 'ka-order-delivery-date-and-time-scheduler' ), $custom_post_type_singular_name),
				/* translators: %s: custom post scheduled update */
				9  => sprintf( __( '%1$s scheduled for: <strong>%2$s</strong>.', 'ka-order-delivery-date-and-time-scheduler' ), $custom_post_type_singular_name, date_i18n( __( 'M j, Y @ G:i' ), strtotime( $post->post_date ) ) ),
				/* translators: %s: custom post draft update */
				10 => sprintf( __( '%s draft updated.', 'ka-order-delivery-date-and-time-scheduler' ), $custom_post_type_singular_name ),
			);

			return $custom_messages;
		}


		/**
		 * Specify custom bulk actions messages for different post types.
		 *
		 * @param  array $bulk_messages Array of messages.
		 * @param  array $bulk_counts Array of how many objects were updated.
		 * @return array
		 */
		public function af_wc_daps_bulk_post_updated_messages( $bulk_messages, $bulk_counts ) {
			$bulk_messages['daps_rules'] = array(
				/* translators: %s: rules count */
				'updated'   => _n( '%s rule updated.', '%s rules updated.', $bulk_counts['updated'], 'ka-order-delivery-date-and-time-scheduler' ),
				/* translators: %s: rules count */
				'locked'    => _n( '%s rule not updated, somebody is editing it.', '%s rules not updated, somebody is editing them.', $bulk_counts['locked'], 'ka-order-delivery-date-and-time-scheduler' ),
				/* translators: %s: rules count */
				'deleted'   => _n( '%s rule permanently deleted.', '%s rules permanently deleted.', $bulk_counts['deleted'], 'ka-order-delivery-date-and-time-scheduler' ),
				/* translators: %s: rules count */
				'trashed'   => _n( '%s rule moved to the Trash.', '%s rules moved to the Trash.', $bulk_counts['trashed'], 'ka-order-delivery-date-and-time-scheduler' ),
				/* translators: %s: rules count */
				'untrashed' => _n( '%s rule restored from the Trash.', '%s rules restored from the Trash.', $bulk_counts['untrashed'], 'ka-order-delivery-date-and-time-scheduler' ),
			);

			$bulk_messages['daps_special_days'] = array(
				/* translators: %s: special days count */
				'updated'   => _n( '%s special day updated.', '%s special days updated.', $bulk_counts['updated'], 'ka-order-delivery-date-and-time-scheduler' ),
				/* translators: %s: special days count */
				'locked'    => _n( '%s special day not updated, somebody is editing it.', '%s special days not updated, somebody is editing them.', $bulk_counts['locked'], 'ka-order-delivery-date-and-time-scheduler' ),
				/* translators: %s: special days count */
				'deleted'   => _n( '%s special day permanently deleted.', '%s special days permanently deleted.', $bulk_counts['deleted'], 'ka-order-delivery-date-and-time-scheduler' ),
				/* translators: %s: special days count */
				'trashed'   => _n( '%s special day moved to the Trash.', '%s special days moved to the Trash.', $bulk_counts['trashed'], 'ka-order-delivery-date-and-time-scheduler' ),
				/* translators: %s: special days count */
				'untrashed' => _n( '%s special day restored from the Trash.', '%s special days restored from the Trash.', $bulk_counts['untrashed'], 'ka-order-delivery-date-and-time-scheduler' ),
			);

			$bulk_messages['daps_holidays'] = array(
				/* translators: %s: holidays count */
				'updated'   => _n( '%s holiday updated.', '%s holidays updated.', $bulk_counts['updated'], 'ka-order-delivery-date-and-time-scheduler' ),
				/* translators: %s: holidays count */
				'locked'    => _n( '%s holiday not updated, somebody is editing it.', '%s holidays not updated, somebody is editing them.', $bulk_counts['locked'], 'ka-order-delivery-date-and-time-scheduler' ),
				/* translators: %s: holidays count */
				'deleted'   => _n( '%s holiday permanently deleted.', '%s holidays permanently deleted.', $bulk_counts['deleted'], 'ka-order-delivery-date-and-time-scheduler' ),
				/* translators: %s: holidays count */
				'trashed'   => _n( '%s holiday moved to the Trash.', '%s holidays moved to the Trash.', $bulk_counts['trashed'], 'ka-order-delivery-date-and-time-scheduler' ),
				/* translators: %s: holidays count */
				'untrashed' => _n( '%s holiday restored from the Trash.', '%s holidays restored from the Trash.', $bulk_counts['untrashed'], 'ka-order-delivery-date-and-time-scheduler' ),
			);

			return $bulk_messages;
		}
	}


	new Ka_Order_Delivery_Date_And_Time_Scheduler();
}








